@@ -15,9 +15,13 @@ import logging
 import logging.handlers
 
 import numpy
+import numpy as np
 from numpy import ma
 
-import pupynere
+try:
+    import netCDF4
+except:
+    import pupynere
 from pydap.client import open_url
 
 # Time to refactor and change somethings. Once class to download, and another
@@ -64,12 +68,17 @@ class AVISO_fetch(object):
         self.set_dataset()
 
         self.file = os.path.join(self.cfg['datadir'], self.cfg['filename'])
-        self.nc = pupynere.netcdf_file(self.file,'w')
+        try:
+            self.nc = netCDF4.Dataset(self.file,'w', format='NETCDF4')
+        except:
+            self.nc = pupynere.netcdf_file(self.file,'w')
 
         self.download_time()
         self.download_LonLat()
         self.download_data()
 
+        self.nc.close()
+
 
     def set_logger(self):
         """
@@ -136,17 +145,28 @@ class AVISO_fetch(object):
         """
         """
         self.logger.debug("Downloading time")
+        t = self.dataset['h']['time'][:]
+        # Improve it and extract this date from the DAP server
+        d0=datetime(1950,1,1)
         if 't_ini' not in self.cfg['limits']:
-            self.cfg['limits']['t_ini'] = 0
-            self.logger.debug("Setting t_ini: %s" % self.cfg['limits']['t_ini'])
+            if 'd_ini' in self.cfg['limits']:
+                d = datetime.strptime(self.cfg['limits']['d_ini'], '%Y-%m-%d')
+                self.cfg['limits']['t_ini'] = np.nonzero(t>=((d-d0).days*24))[0][0]
+            else:
+                self.cfg['limits']['t_ini'] = 0
+                self.logger.debug("Setting t_ini: %s" % self.cfg['limits']['t_ini'])
 
         if 't_step' not in self.cfg['limits']:
             self.cfg['limits']['t_step'] = 1
             self.logger.debug("Setting t_step: %s" % self.cfg['limits']['t_step'])
 
         if 't_fin' not in self.cfg['limits']:
-            self.cfg['limits']['t_fin'] = self.dataset['h']['time'].shape[0]
-            self.logger.debug("Setting t_fin: %s" % self.cfg['limits']['t_fin'])
+            if 'd_fin' in self.cfg['limits']:
+                d = datetime.strptime(self.cfg['limits']['d_fin'], '%Y-%m-%d')
+                self.cfg['limits']['t_fin'] = np.nonzero(t>((d-d0).days*24))[0][0]
+            else:
+                self.cfg['limits']['t_fin'] = self.dataset['h']['time'].shape[0]
+                self.logger.debug("Setting t_fin: %s" % self.cfg['limits']['t_fin'])
 
         t_ini = self.cfg['limits']['t_ini']
         t_fin = self.cfg['limits']['t_fin']
@@ -176,27 +196,42 @@ class AVISO_fetch(object):
         self.logger.debug("Downloading LonLat")
         data = {}
         limits = self.cfg['limits']
-        Lat = self.dataset['h']['NbLatitudes']
-        Lon = self.dataset['h']['NbLongitudes']
+        Lat = self.dataset['h']['NbLatitudes'][:].astype('f')
+        Lon = self.dataset['h']['NbLongitudes'][:].astype('f')
+
+        # If data is requested as -180/180, convert to 0/360,
+        #   which is the pattern in AVISO
+        if limits['lonini'] < 0: limits['lonini']+=360
+        if limits['lonfin'] < 0: limits['lonfin']+=360
 
         Latlimits = numpy.arange(Lat.shape[0])[(Lat[:]>=limits["latini"]) & (Lat[:]<=limits["latfin"])]
         Latlimits = [Latlimits[0],Latlimits[-1]]
 
-        Lonlimits = numpy.arange(Lon.shape[0])[(Lon[:]>=limits["lonini"]) & (Lon[:]<=limits["lonfin"])]
-        Lonlimits=[Lonlimits[0],Lonlimits[-1]]
+        lat = Lat[Latlimits[0]:Latlimits[-1]]
+
+        if limits['lonfin'] > limits['lonini']:
+            Lonlimits = numpy.arange(Lon.shape[0])[(Lon[:]>=limits["lonini"]) & (Lon[:]<=limits["lonfin"])]
+            Lonlimits=[Lonlimits[0],Lonlimits[-1]]
+
+            lon = Lon[Lonlimits[0]:Lonlimits[-1]]
+        else:
+            Lonlimits = [np.nonzero(Lon>=limits['lonini'])[0][0], 
+                    np.nonzero(Lon<=limits['lonfin'])[0][-1]+1] 
+
+            lon = np.append(Lon[Lonlimits[0]:],Lon[:Lonlimits[1]])
+                    
 
         self.cfg['limits']['Latlimits'] = Latlimits
         self.cfg['limits']['Lonlimits'] = Lonlimits
 
-        Lon, Lat = numpy.meshgrid( (Lon[Lonlimits[0]:Lonlimits[-1]]), (Lat[Latlimits[0]:Latlimits[-1]]) )
 
+        Lon, Lat = numpy.meshgrid( lon, lat )
 
-        self.slice_size = (Lonlimits[-1]-Lonlimits[0])* \
-                    (Latlimits[-1]-Latlimits[0])
+        self.slice_size = lon.shape[0]*lat.shape[0]
 
         # ========
-        self.nc.createDimension('lat', (Latlimits[-1]-Latlimits[0]))
-        self.nc.createDimension('lon', (Lonlimits[-1]-Lonlimits[0]))
+        self.nc.createDimension('lat', lat.shape[0])
+        self.nc.createDimension('lon', lon.shape[0])
 
         ncLat = self.nc.createVariable('Lat', 'f', ('lat', 'lon'))
         ncLon = self.nc.createVariable('Lon', 'f', ('lat', 'lon'))
@@ -246,7 +281,13 @@ class AVISO_fetch(object):
                 for i in range(ntries):
                     print "Try n: %s" % i
                     try:
-                        data[v][ind] = dataset[b1:b2:self.cfg['limits']['t_step'], Lonlimits[0]:Lonlimits[-1],Latlimits[0]:Latlimits[-1]].swapaxes(1,2).astype('f')
+                        if Lonlimits[1] > Lonlimits[0]:
+                            tmp = dataset[b1:b2:self.cfg['limits']['t_step'], Lonlimits[0]:Lonlimits[-1],Latlimits[0]:Latlimits[-1]]
+                        else:
+                            tmp1 = dataset[b1:b2:self.cfg['limits']['t_step'], Lonlimits[0]:,Latlimits[0]:Latlimits[-1]]
+                            tmp2 = dataset[b1:b2:self.cfg['limits']['t_step'], :Lonlimits[-1],Latlimits[0]:Latlimits[-1]]
+                            tmp = np.append(tmp1, tmp2, axis=1)
+                        data[v][ind] = tmp.swapaxes(1,2).astype('f')
                         break
                     except:
                         waitingtime = 30+i*20
@@ -255,6 +296,27 @@ class AVISO_fetch(object):
             #data['h'] = 1e-2*data['h'].swapaxes(1,2)
 
 
+class Products(object):
+    """ Calculate some products from the downloaded data
+    """
+    def __init__(self, cfg):
+        self.cfg = cfg
+        self.file = os.path.join(self.cfg['datadir'], self.cfg['filename'])
+        self.nc = pupynere.netcdf_file(self.file,'a')
+        for tn in range(nc.dimensions['time']):
+            data = {'t': datetime.strptime(nc.variables['time'].units, 
+                'hours since %Y-%m-%d') + timedelta(hours=float(nc.variables['time'][tn])), 
+                'Lat': ma.array(nc.variables['Lat'][:]),
+                'Lon': ma.array(nc.variables['Lon'][:]), 
+                'u': ma.masked_values(nc.variables['u'][tn],
+                    nc.variables['u'].missing_value),
+                'v': ma.masked_values(nc.variables['v'][tn], 
+                    nc.variables['v'].missing_value),
+                }
+
+#products = okuboweiss.okuboweiss(data)
+
+#products = okuboweiss.OkuboWeiss(data, cfg['okuboweiss']) %, logname = metadata['log']['logname'])
 
 
 
